; (c) Copyright 1989 Murphy Software.  All Rights Reserved.
;
; Unauthorized duplication or use prohibited.

; ********************* DEMO.SC **********************************
; Example of PDShell usage for reading a serial port through the
; keyboard buffer and putting the results into a field of a table.
;*****************************************************************

;************* YesNo ***************
PROC YesNo(Q)
; Display the question Q as a message, get response and return true or false.

PRIVATE Response, C

STYLE ATTRIBUTE 112
Q = Q + " [Y,N]    "
@22,1 CLEAR EOL
@22,80-LEN(Q)
?? Q
@22,77
WHILE TRUE
  C = GETCHAR()
  C = CHR(C)
  SWITCH
    CASE "Y" = UPPER(C):
      ?? "Yes"
      SLEEP 500
      Response = TRUE
      QUITLOOP
    CASE "N" = UPPER(C):
      ?? "No"
      SLEEP 500
      Response = FALSE
      QUITLOOP
    OTHERWISE : BEEP
  ENDSWITCH
ENDWHILE
STYLE
@22,0 CLEAR EOL
RETURN Response
ENDPROC ;YesNo

;************* PDShellCmd **********
PROC PDShellCmd(Cmd)
  ; Send a message to PDShell if it is present. PDShell is looking at top left of
  ; screen for messages.  Character PDShellCmdStart signals that a message is
  ; present. PDShell signals completion by poking a chr(249) into kbd buffer.

  PRIVATE Response, Waited, PDShellCmdStart, PDShellCmdEnd

  PDShellCmdStart = CHR(18)          ; PDShell Cmd characters
  PDShellCmdEnd   = CHR(20)          ;

  @0,0 ?? PDShellCmdStart + Cmd + PDShellCmdEnd
  Response = 0
  Waited = 0
  WHILE Waited < 2.0       ; Wait for 2 seconds
    Response = GETCHAR()
    IF Response = 249
      THEN QUITLOOP
    ENDIF
    SLEEP(100)
    Waited = Waited + 0.1
  ENDWHILE
  @0,0 CLEAR EOL
ENDPROC  ; PDShellCmd

;******************* ReadScale *****************
PROC ReadScale()
; This procedure reads the serial port connected to a scale
; using ACCEPT.  A picture is specified to allows the input of only a stable
; reading.  The scale in this case continuously sends measurements in the
; format indicated. An unstable measurment is shown by a 'US' at the start
; of the input line.  The picture format used here waits for a measurement
; started with a 'ST'.  A good input is then parsed, the weight extracted
; and then the weight returned. If the scale were to be turned off, the
; user could escape from the measurment by typing escape.  In this case,
; the weight returned would be 0.
;
; Note that because scale status is verified using the picture method,
; that the displayed Scale Reading could look garbled at times.  As long
; as the scale always sends CRLF at the end of a measurement, there should
; be no possibility of lock up.


PRIVATE Lbs, WStr
  PDShellCmd("COn")          ; Tell PDShell to route COMn input to kbd buffer
  Lbs = 0                    ; Init Lbs to 0
  WHILE (TRUE)

    @0,0 CLEAR EOL
    @1,0 CLEAR EOL
    @1,0 ??"Format:        ST,+####.#lb"
    WStr = ""

    @0,0 ??"SCALE READING: "
    ACCEPT "A12"
                             ; See PAL manual on pictures for explanation
;      PICTURE "{S}{T}{;,}{+}####{.}#{l}{b}"
    TO WStr

    IF RETVAL = FALSE        ; In case scale turned off, user must hit "Esc"
      THEN QUITLOOP          ; to exit this loop
    ENDIF
                             ; Alternative to picture. Check first 4 chars
                             ; for valid weight observation.
    IF "ST,+" = SUBSTR(WStr,1,4) THEN
      Lbs = NUMVAL(SUBSTR(WStr,5,6))  ; Parse weight from string
      IF Lbs > 0             ; If weight is greater than 0, we got a good one
        THEN QUITLOOP
      ENDIF
    ENDIF
  ENDWHILE
  @1,0 CLEAR EOL
  PDShellCmd("COff")         ; Tell PDShell to stop sending COMn input to kbd

  WHILE CHARWAITING()        ; Clear keyboard buffer
    C=GETCHAR()
  ENDWHILE ;
  RETURN Lbs
ENDPROC ; ReadScale

PROC GetStream()
; Turn on serial port and get characters from the stream until an 'X' is input
PRIVATE C
  CLEAR
  C = 27
  PDShellCmd("COn")          ; Tell PDShell to route COMn input to kbd buffer
  @0,0 ??"Type in characters. Exit with 'X'."
  @2,0
  WHILE CHR(C) <> "X"
    C = GETCHAR()
    IF (C <= 255) AND (C >= 1) THEN ??CHR(C) ELSE C = 1 ENDIF
  ENDWHILE
  CLEAR
  PDShellCmd("COff")         ; Tell PDShell to stop sending COMn input to kbd
ENDPROC ;GetStream

;**************************** Main ********************************
WHILE TRUE
  CLEAR
  SHOWMENU
    "ReadScale" : "Read from the scale",
    "GetStream" : "Get and show stream of characters from serial port",
    "Quit"      : "Quit the demonstration"
  TO Option
  SWITCH
    CASE Option = "ReadScale" :
      MESSAGE "Scale read " + STRVAL(ReadScale())
    CASE Option = "GetStream" :
      GetStream()
    CASE Option = "Quit" :
      QUITLOOP
  ENDSWITCH
ENDWHILE
